@@ -5,22 +5,16 @@ Note that the tests can take few minutes to run because of the time
 required to bring up/down the VM.
 
 Most test functions (decorated with `@with_setup`) will actually bring the VM
-up/down. This for one is the "proper" way of doing things (isolation).  The
-downside of such workflow is that it increases the execution time of the test
-suite. The other approach (bringing the VM up once and packing all tests
-together) is less extensible.  With few tests it may work, but when the test
-suite grows it becomes a problem.
-
-In order to mitigate the inconvenience, each test method actually
-encapsulates few related tests. Aside from making the execution time shorter
-it also adds to readability.
+up/down. This is the "proper" way of doing things (isolation).  However, the
+downside of such a workflow is that it increases the execution time of the test
+suite.
 
 Before the first test a base box is added to Vagrant under the name
 TEST_BOX_NAME. This box is not deleted after the test suite runs in order
 to avoid downloading of the box file on every run.
 '''
 
-
+from __future__ import print_function
 import os
 import re
 import unittest
@@ -29,9 +23,10 @@ import subprocess
 import sys
 import tempfile
 import time
-from nose.tools import eq_, with_setup
+from nose.tools import eq_, ok_, with_setup
 
 import vagrant
+from vagrant import compat
 
 # location of a test file on the created box by provisioning in vm_Vagrantfile
 TEST_FILE_PATH = '/home/vagrant/python_vagrant_test_file'
@@ -55,34 +50,24 @@ TD = None
 
 
 
-def list_boxes():
-    '''
-    Return a list if tuples containing the box name and box provider of each
-    box in `vagrant box list`.
-    '''
-    # Example `box list` output (vagrant 1.1.5):
-    # base                     (virtualbox)
-    # precise32                (virtualbox)
-    # precise64                (virtualbox)
-    # python-vagrant-base      (virtualbox)
-    # python-vagrant-dummy-box (virtualbox)
-    # Example `box list` output with no boxes (vagrant 1.2.7):
-    # There are no installed boxes! Use `vagrant box add` to add some.
-    listing = subprocess.check_output('vagrant box list', shell=True)
-    boxes = []
-    for line in listing.splitlines():
-        m = re.search(r'^\s*(?P<name>.+?)\s+\((?P<provider>[^)]+)\)\s*$', line)
-        if m:
-            print m.groups()
-            boxes.append((m.group('name'), m.group('provider')))
-    return boxes
-
-
 def list_box_names():
     '''
-    Return a list of the currently installed vagrant box names.
+    Return a list of the currently installed vagrant box names.  This is 
+    implemented outside of `vagrant.Vagrant`, so that it will still work
+    even if the `Vagrant.box_list()` implementation is broken.
     '''
-    return [box[0] for box in list_boxes()]
+    listing = compat.decode(subprocess.check_output('vagrant box list --machine-readable', shell=True))
+    box_names = []
+    for line in listing.splitlines():
+        # Vagrant 1.8 added additional fields to the --machine-readable output,
+        # so unpack the fields according to the number of separators found.
+        if line.count(',') == 3:
+            timestamp, _, kind, data = line.split(',')
+        else:
+            timestamp, _, kind, data, extra_data = line.split(',')
+        if kind == 'box-name':
+            box_names.append(data.strip())
+    return box_names
 
 
 # MODULE-LEVEL SETUP AND TEARDOWN
@@ -153,18 +138,91 @@ def teardown_vm():
 
 
 @with_setup(make_setup_vm(), teardown_vm)
-def test_plugin_list_parsing():
+def test_parse_plugin_list():
     '''
     Test the parsing the output of the `vagrant plugin list` command.
     '''
     # listing should match output generated by `vagrant plugin list`.
-    listing = '''sahara (0.0.16)
-vagrant-login (1.0.1, system)
+    listing = '''1424145521,,plugin-name,sahara
+1424145521,sahara,plugin-version,0.0.16
+1424145521,,plugin-name,vagrant-share
+1424145521,vagrant-share,plugin-version,1.1.3%!(VAGRANT_COMMA) system
 '''
     # Can compare tuples to Plugin class b/c Plugin is a collections.namedtuple.
-    goal = [('sahara', '0.0.16', False), ('vagrant-login', '1.0.1', True)]
+    goal = [('sahara', '0.0.16', False), ('vagrant-share', '1.1.3', True)]
     v = vagrant.Vagrant(TD)
-    parsed = [v._parse_plugin_list_line(l) for l in listing.splitlines()]
+    parsed = v._parse_plugin_list(listing)
+    assert goal == parsed, 'The parsing of the test listing did not match the goal.\nlisting={!r}\ngoal={!r}\nparsed_listing={!r}'.format(listing, goal, parsed)
+
+
+@with_setup(make_setup_vm(), teardown_vm)
+def test_parse_box_list():
+    '''
+    Test the parsing the output of the `vagrant box list` command.
+    '''
+    listing = '''1424141572,,box-name,precise64
+1424141572,,box-provider,virtualbox
+1424141572,,box-version,0
+1424141572,,box-name,python-vagrant-base
+1424141572,,box-provider,virtualbox
+1424141572,,box-version,0
+'''
+    # Can compare tuples to Box class b/c Box is a collections.namedtuple.
+    goal = [('precise64', 'virtualbox', '0'),
+            ('python-vagrant-base', 'virtualbox', '0')]
+    v = vagrant.Vagrant(TD)
+    parsed = v._parse_box_list(listing)
+    assert goal == parsed, 'The parsing of the test listing did not match the goal.\nlisting={!r}\ngoal={!r}\nparsed_listing={!r}'.format(listing, goal, parsed)
+
+
+@with_setup(make_setup_vm(), teardown_vm)
+def test_parse_status():
+    '''
+    Test the parsing the output of the `vagrant status` command.
+    '''
+    listing = '''1424098924,web,provider-name,virtualbox
+1424098924,web,state,running
+1424098924,web,state-human-short,running
+1424098924,web,state-human-long,The VM is running. To stop this VM%!(VAGRANT_COMMA) you can run `vagrant halt` to\\nshut it down forcefully%!(VAGRANT_COMMA) or you can run `vagrant suspend` to simply\\nsuspend the virtual machine. In either case%!(VAGRANT_COMMA) to restart it again%!(VAGRANT_COMMA)\\nsimply run `vagrant up`.
+1424098924,db,provider-name,virtualbox
+1424098924,db,state,not_created
+1424098924,db,state-human-short,not created
+1424098924,db,state-human-long,The environment has not yet been created. Run `vagrant up` to\\ncreate the environment. If a machine is not created%!(VAGRANT_COMMA) only the\\ndefault provider will be shown. So if a provider is not listed%!(VAGRANT_COMMA)\\nthen the machine is not created for that environment.
+'''
+    # Can compare tuples to Status class b/c Status is a collections.namedtuple.
+    goal = [('web', 'running', 'virtualbox'),
+            ('db', 'not_created', 'virtualbox')]
+    v = vagrant.Vagrant(TD)
+    parsed = v._parse_status(listing)
+    assert goal == parsed, 'The parsing of the test listing did not match the goal.\nlisting={!r}\ngoal={!r}\nparsed_listing={!r}'.format(listing, goal, parsed)
+
+
+@with_setup(make_setup_vm(), teardown_vm)
+def test_parse_aws_status():
+    '''
+    Test the parsing the output of the `vagrant status` command for an aws instance.
+    '''
+    listing = '''1462351212,default,action,read_state,start
+1462351214,default,action,read_state,end
+1462351214,default,metadata,provider,aws
+1462351214,default,action,read_state,start
+1462351215,default,action,read_state,end
+1462351215,default,action,read_state,start
+1462351216,default,action,read_state,end
+1462351216,default,action,read_state,start
+1462351217,default,action,read_state,end
+1462351217,default,provider-name,aws
+1462351217,default,state,running
+1462351217,default,state-human-short,running
+1462351217,default,state-human-long,The EC2 instance is running. To stop this machine%!(VAGRANT_COMMA) you can run\\n`vagrant halt`. To destroy the machine%!(VAGRANT_COMMA) you can run `vagrant destroy`.
+1462351217,default,action,read_state,start
+1462351219,default,action,read_state,end
+1462351219,,ui,info,Current machine states:\\n\\ndefault (aws)\\n\\nThe EC2 instance is running. To stop this machine%!(VAGRANT_COMMA) you can run\\n`vagrant halt`. To destroy the machine%!(VAGRANT_COMMA) you can run `vagrant destroy`.
+'''
+    # Can compare tuples to Status class b/c Status is a collections.namedtuple.
+    goal = [('default', 'running', 'aws')]
+    v = vagrant.Vagrant(TD)
+    parsed = v._parse_status(listing)
     assert goal == parsed, 'The parsing of the test listing did not match the goal.\nlisting={!r}\ngoal={!r}\nparsed_listing={!r}'.format(listing, goal, parsed)
 
 
@@ -223,7 +281,7 @@ def test_vm_config():
     v = vagrant.Vagrant(TD)
     v.up()
     command = "vagrant ssh-config"
-    ssh_config = subprocess.check_output(command, cwd=TD, shell=True)
+    ssh_config = compat.decode(subprocess.check_output(command, cwd=TD, shell=True))
     parsed_config = dict(line.strip().split(None, 1) for line in
                             ssh_config.splitlines() if line.strip() and not
                             line.strip().startswith('#'))
@@ -248,7 +306,11 @@ def test_vm_config():
         "{}@{}:{}".format(expected_user, expected_hostname, expected_port))
 
     keyfile = v.keyfile()
-    eq_(keyfile, parsed_config["IdentityFile"])
+    try:
+        eq_(keyfile, parsed_config["IdentityFile"])
+    except AssertionError:
+        # Vagrant 1.8 adds quotes around the filepath for the private key.
+        eq_(keyfile, parsed_config["IdentityFile"].lstrip('"').rstrip('"'))
 
 
 @with_setup(make_setup_vm(), teardown_vm)
@@ -292,36 +354,36 @@ def test_vm_sandbox_mode():
     assert sandbox_status == "on", "After bringing the VM up again the status should be 'on', " + "got:'{}'".format(sandbox_status)
 
     test_file_contents = _read_test_file(v)
-    print test_file_contents
+    print(test_file_contents)
     eq_(test_file_contents, None, "There should be no test file")
 
     _write_test_file(v, "foo")
     test_file_contents = _read_test_file(v)
-    print test_file_contents
+    print(test_file_contents)
     eq_(test_file_contents, "foo", "The test file should read 'foo'")
 
     v.sandbox_rollback()
     time.sleep(10)  # https://github.com/jedi4ever/sahara/issues/16
 
     test_file_contents = _read_test_file(v)
-    print test_file_contents
+    print(test_file_contents)
     eq_(test_file_contents, None, "There should be no test file")
 
     _write_test_file(v, "foo")
     test_file_contents = _read_test_file(v)
-    print test_file_contents
+    print(test_file_contents)
     eq_(test_file_contents, "foo", "The test file should read 'foo'")
     v.sandbox_commit()
     _write_test_file(v, "bar")
     test_file_contents = _read_test_file(v)
-    print test_file_contents
+    print(test_file_contents)
     eq_(test_file_contents, "bar", "The test file should read 'bar'")
 
     v.sandbox_rollback()
     time.sleep(10)  # https://github.com/jedi4ever/sahara/issues/16
 
     test_file_contents = _read_test_file(v)
-    print test_file_contents
+    print(test_file_contents)
     eq_(test_file_contents, "foo", "The test file should read 'foo'")
 
     sandbox_status = v._parse_vagrant_sandbox_status("Usage: ...")
@@ -344,35 +406,27 @@ def test_boxes():
     provider = "virtualbox"
 
     # Start fresh with no dummy box
-    boxes = list_boxes()
-    if (box_name, provider) in boxes:
-        subprocess.check_call(
-            'vagrant box remove {} {}'.format(box_name, provider), shell=True)
+    if box_name in list_box_names():
+        subprocess.check_call(['vagrant', 'box', 'remove', box_name])
 
-    # Test that there is no dummy box
-    boxes = list_boxes()
-    eq_((box_name, provider) in boxes, False,
-        "There should be no dummy box before it's added")
+    # Test that there is no dummy box listed
+    ok_(box_name not in [b.name for b in v.box_list()], "There should be no dummy box before it's added.")
 
-    # Test adding a box
+    # Add a box
     v.box_add(box_name, TEST_BOX_URL)
-    boxes = list_boxes()
-    eq_((box_name, provider) in boxes, True,
-        "There should be a dummy box in the list of boxes")
 
-    # Test listing boxes
-    boxes = list_boxes()
-    reported_boxes = [(b.name, b.provider) for b in v.box_list()]
-    for box in boxes:
-        eq_(box in reported_boxes, True,
-            "The box '{}' should be in the list returned by box_list()".format(box))
+    # Test that there is a dummy box listed
+    box_listing = v.box_list()
+    ok_((box_name, provider) in [(b.name, b.provider) for b in box_listing],
+        'The box {box} for provider {provider} should be in the list returned by box_list(). box_list()={box_listing}'.format(
+            box=box_name, provider=provider, box_listing=box_listing))
 
-    # Test removing a box with name and provider
+
+    # Remove dummy box using a box name and provider
     v.box_remove(box_name, provider)
-    boxes = list_boxes()
-    eq_((box_name, provider) in boxes, False,
-        "There should be no dummy box after it's been deleted")
 
+    # Test that there is no dummy box listed
+    ok_(box_name not in [b.name for b in v.box_list()], "There should be no dummy box after it has been removed.")
 
 @with_setup(make_setup_vm(SHELL_PROVISION_VAGRANTFILE), teardown_vm)
 def test_provisioning():
@@ -388,7 +442,7 @@ def test_provisioning():
 
     v.provision()
     test_file_contents = _read_test_file(v)
-    print "Contents: {}".format(test_file_contents)
+    print("Contents: {}".format(test_file_contents))
     eq_(test_file_contents, "foo", "The test file should contain 'foo'")
 
 
@@ -434,7 +488,7 @@ def test_multivm_config():
     v = vagrant.Vagrant(TD, quiet_stdout=False, quiet_stderr=False)
     v.up(vm_name=VM_1)
     command = "vagrant ssh-config " + VM_1
-    ssh_config = subprocess.check_output(command, cwd=TD, shell=True)
+    ssh_config = compat.decode(subprocess.check_output(command, cwd=TD, shell=True))
     parsed_config = dict(line.strip().split(None, 1) for line in
                             ssh_config.splitlines() if line.strip() and not
                             line.strip().startswith('#'))
@@ -459,7 +513,32 @@ def test_multivm_config():
         "{}@{}:{}".format(expected_user, expected_hostname, expected_port))
 
     keyfile = v.keyfile(vm_name=VM_1)
-    eq_(keyfile, parsed_config["IdentityFile"])
+    try:
+        eq_(keyfile, parsed_config["IdentityFile"])
+    except AssertionError:
+        # Vagrant 1.8 adds quotes around the filepath for the private key.
+        eq_(keyfile, parsed_config["IdentityFile"].lstrip('"').rstrip('"'))
+
+
+def test_make_file_cm():
+    filename = os.path.join(TD, 'test.log')
+    if os.path.exists(filename):
+        os.remove(filename)
+
+    # Test writing to the filehandle yielded by cm
+    cm = vagrant.make_file_cm(filename)
+    with cm() as fh:
+        fh.write('one\n')
+
+    with open(filename) as read_fh:
+        assert read_fh.read() == 'one\n'
+
+    # Test appending to the file yielded by cm
+    with cm() as fh:
+        fh.write('two\n')
+
+    with open(filename) as read_fh:
+        assert read_fh.read() == 'one\ntwo\n'
 
 
 def _execute_command_in_vm(v, command):
@@ -474,7 +553,7 @@ def _execute_command_in_vm(v, command):
 
     # ignore the fact that this host is not in our known hosts
     ssh_command = [vagrant_exe, 'ssh', '-c', command]
-    return subprocess.check_output(ssh_command, cwd=v.root)
+    return compat.decode(subprocess.check_output(ssh_command, cwd=v.root))
 
 
 def _write_test_file(v, file_contents):
